@@ -1,6 +1,6 @@
 # vim:fileencoding=utf8
 
-# Copyright (c) 2006-2017  Andrey Golovigin
+# Copyright (c) 2006-2019  Andrey Golovigin
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -36,39 +36,23 @@ Inspired by Brevé -- http://breve.twisty-industries.com/
 >>> book_format = sentence(capfirst=True, sep=', ') [
 ...     field('title'), field('year'), optional [field('sdf')]
 ... ]
->>> print(str(book_format.format_data(e)))
+>>> print(six.text_type(book_format.format_data({'entry': e})))
 The Book, 2000.
->>> print(str(words ['one', 'two', words ['three', 'four']].format_data(e)))
+>>> print(six.text_type(words ['one', 'two', words ['three', 'four']].format_data(e)))
 one two three four
 """
 
+from __future__ import unicode_literals
+
+import  six
+
 from pybtex import richtext
 from pybtex.exceptions import PybtexError
+from pybtex.py3compat import fix_unicode_literals_in_doctest
 
 __test__ = {}  # for doctest
 
 
-class Proto(object):
-    def __init__(self, *args, **kwargs):
-        self.args = args
-        self.kwargs = kwargs
-
-    def __call__(self, *args, **kwargs):
-        return Node(*self.args, **self.kwargs)(*args, **kwargs)
-
-    def __getitem__(self, children):
-        return Node(*self.args, **self.kwargs)[children]
-
-    def __repr__(self):
-        return repr(Node(*self.args, **self.kwargs))
-
-    def format_data(self, *args, **kwargs):
-        return Node(*self.args, **self.kwargs).format_data(*args, **kwargs)
-
-    def format(self):
-        return self.format_data(None)
-
-
 class Node(object):
     def __init__(self, name, f):
         self.name = name
@@ -77,34 +61,44 @@ class Node(object):
         self.kwargs = {}
         self.children = []
 
+    def _clone(self):
+        result = Node(self.name, self.f)
+        result.args = list(self.args)
+        result.kwargs = dict(self.kwargs)
+        result.children = list(self.children)
+        return result
+
     def __call__(self, *args, **kwargs):
-        self.args = args
-        self.kwargs.update(kwargs)
-        return self
+        result = self._clone()
+        result.args.extend(args)
+        result.kwargs.update(kwargs)
+        return result
 
     def __getitem__(self, children):
+        result = self._clone()
         if isinstance(children, (list, tuple)):
-            self.children = children
+            result.children.extend(children)
         else:
-            self.children.append(children)
-        return self
+            result.children.append(children)
+        return result
 
+    @fix_unicode_literals_in_doctest
     def __repr__(self):
         """
         >>> join(', ')
-        join(', ')
+        join(u', ')
         >>> join
         join
         >>> join ['a']
-        join ['a']
+        join [u'a']
         >>> join ['a', 'b', 'c']
-        join ['a', 'b', 'c']
-        >>> join(' ') ['a', 'b', 'c']
-        join(' ') ['a', 'b', 'c']
-        >>> join(sep=' ') ['a', 'b', 'c']
-        join(sep=' ') ['a', 'b', 'c']
-        >>> join(sep=' ') [tag('em') ['a', 'b', 'c']]
-        join(sep=' ') [tag('em') ['a', 'b', 'c']]
+        join [u'a', u'b', u'c']
+        >>> join(' ') [u'a', u'b', u'c']
+        join(u' ') [u'a', u'b', u'c']
+        >>> join(sep=' ') [u'a', u'b', u'c']
+        join(sep=u' ') [u'a', u'b', u'c']
+        >>> join(sep=u' ') [tag('em') [u'a', u'b', u'c']]
+        join(sep=u' ') [tag(u'em') [u'a', u'b', u'c']]
 
         """
         params = []
@@ -112,7 +106,7 @@ class Node(object):
         if args_repr:
             params.append(args_repr)
         kwargs_repr = ', '.join(
-            '%s=%s' % (key, repr(value)) for (key, value) in iter(self.kwargs.items())
+            '%s=%s' % (key, repr(value)) for (key, value) in self.kwargs.items()
         )
         if kwargs_repr:
             params.append(kwargs_repr)
@@ -159,19 +153,19 @@ def _format_list(list_, data):
 def node(f):
     if f.__doc__:
         __test__[f.__name__] = f
-    return Proto(f.__name__, f)
+    return Node(f.__name__, f)
 
 
 @node
 def join(children, data, sep='', sep2=None, last_sep=None):
     """Join text fragments together.
-    >>> print(str(join.format()))
+    >>> print(six.text_type(join.format()))
     <BLANKLINE>
-    >>> print(str(join ['a', 'b', 'c', 'd', 'e'].format()))
+    >>> print(six.text_type(join ['a', 'b', 'c', 'd', 'e'].format()))
     abcde
-    >>> print(str(join(sep=', ', sep2=' and ', last_sep=', and ') ['Tom', 'Jerry'].format()))
+    >>> print(six.text_type(join(sep=', ', sep2=' and ', last_sep=', and ') ['Tom', 'Jerry'].format()))
     Tom and Jerry
-    >>> print(str(join(sep=', ', sep2=' and ', last_sep=', and ') ['Billy', 'Willy', 'Dilly'].format()))
+    >>> print(six.text_type(join(sep=', ', sep2=' and ', last_sep=', and ') ['Billy', 'Willy', 'Dilly'].format()))
     Billy, Willy, and Dilly
     """
 
@@ -200,15 +194,15 @@ def together(children, data, last_tie=False):
     """
     Try to keep words together, like BibTeX does.
 
-    >>> print(str(together ['very', 'long', 'road'].format()))
+    >>> print(six.text_type(together ['very', 'long', 'road'].format()))
     very long road
-    >>> print(str(together(last_tie=True) ['very', 'long', 'road'].format()))
+    >>> print(six.text_type(together(last_tie=True) ['very', 'long', 'road'].format()))
     very long<nbsp>road
-    >>> print(str(together ['a', 'very', 'long', 'road'].format()))
+    >>> print(six.text_type(together ['a', 'very', 'long', 'road'].format()))
     a<nbsp>very long road
-    >>> print(str(together ['chapter', '8'].format()))
+    >>> print(six.text_type(together ['chapter', '8'].format()))
     chapter<nbsp>8
-    >>> print(str(together ['chapter', '666'].format()))
+    >>> print(six.text_type(together ['chapter', '666'].format()))
     chapter 666
     """
     from pybtex.textutils import tie_or_space
@@ -232,11 +226,11 @@ def together(children, data, last_tie=False):
 def sentence(children, data, capfirst=False, capitalize=False, add_period=True, sep=', '):
     """Join text fragments, capitalyze the first letter, add a period to the end.
 
-    >>> print(str(sentence.format()))
+    >>> print(six.text_type(sentence.format()))
     <BLANKLINE>
-    >>> print(str(sentence(capitalize=True, sep=' ') ['mary', 'had', 'a', 'little', 'lamb'].format()))
+    >>> print(six.text_type(sentence(capitalize=True, sep=' ') ['mary', 'had', 'a', 'little', 'lamb'].format()))
     Mary had a little lamb.
-    >>> print(str(sentence(capitalize=False, add_period=False) ['uno', 'dos', 'tres'].format()))
+    >>> print(six.text_type(sentence(capitalize=False, add_period=False) ['uno', 'dos', 'tres'].format()))
     uno, dos, tres
     """
 
@@ -254,18 +248,21 @@ class FieldIsMissing(PybtexError):
     def __init__(self, field_name, entry):
         self.field_name = field_name
         super(FieldIsMissing, self).__init__(
-            'missing {0} in {1}'.format(field_name, getattr(entry, 'key', '<unnamed>'))
+            u'missing {0} in {1}'.format(field_name, getattr(entry, 'key', '<unnamed>'))
         )
 
 @node
-def field(children, data, name, apply_func=None, raw=False):
+def field(children, context, name, apply_func=None, raw=False):
     """Return the contents of the bibliography entry field."""
 
     assert not children
+    entry = context['entry']
     try:
-        field = data.fields[name] if raw else data.rich_fields[name]
+        field = entry._find_field(name, bib_data=context.get('bib_data'))
+        if not raw:
+            field = richtext.Text.from_latex(field)
     except KeyError:
-        raise FieldIsMissing(name, data)
+        raise FieldIsMissing(name, entry)
     else:
         if apply_func:
             field = apply_func(field)
@@ -273,20 +270,19 @@ def field(children, data, name, apply_func=None, raw=False):
 
 
 @node
-def names(children, data, role, **kwargs):
+def names(children, context, role, **kwargs):
     """Return formatted names."""
 
     assert not children
-    try:
-        persons = data.persons[role]
-    except KeyError:
-        # role is a bibtex field so it makes sense
-        # to raise FieldIsMissing; optional will catch it
-        raise FieldIsMissing(role, data)
 
-    # person.text is set by pybtex.style.formatting.BaseStyle.format_entries()
-    # TODO: do something about it, because it is confusing
-    return join(**kwargs) [[person.text for person in persons]].format_data(data)
+    try:
+        persons = context['entry'].persons[role]
+    except KeyError:
+        raise FieldIsMissing(role, context['entry'])
+
+    style = context['style']
+    formatted_names = [style.format_name(person, style.abbreviate_names) for person in persons]
+    return join(**kwargs) [formatted_names].format_data(context)
 
 
 @node
@@ -296,7 +292,7 @@ def optional(children, data):
 
     >>> from pybtex.database import Entry
     >>> template = optional [field('volume'), optional['(', field('number'), ')']]
-    >>> template.format_data(Entry('article'))
+    >>> template.format_data({'entry': Entry('article')})
     Text()
 
     """
@@ -350,4 +346,3 @@ def first_of(children, data):
         if child:
             return child
     return richtext.Text()
-
