@@ -1,4 +1,4 @@
-# Copyright (c) 2006-2017  Andrey Golovigin
+# Copyright (c) 2006-2019  Andrey Golovigin
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -21,20 +21,26 @@
 
 
 """Miscellaneous small utils."""
+from __future__ import print_function, unicode_literals
 
 import itertools
+from collections import (
+    MutableMapping, MutableSet, OrderedDict, Sequence, deque
+)
 from functools import wraps
-from collections import deque, Sequence, MutableMapping, MutableSet
-from collections import OrderedDict
 from types import GeneratorType
 
+from six.moves import zip_longest
+
+from .py3compat import fix_unicode_literals_in_doctest
+
 
 def deprecated(since, reason=None):
     def decorator(f):
         @wraps(f)
         def new_f(*args, **kwargs):
             import warnings
-            message = '{0}() is deprecated since {1}'.format(f.__name__, since)
+            message = u'{0}() is deprecated since {1}'.format(f.__name__, since)
             if reason:
                 message += ': {0}'.format(reason)
             warnings.warn(message, DeprecationWarning, stacklevel=2)
@@ -68,17 +74,18 @@ def collect_iterable(f):
 def pairwise(iterable):
     a, b = itertools.tee(iterable)
     next(b, None)
-    return itertools.zip_longest(a, b)
+    return zip_longest(a, b)
 
 
+@fix_unicode_literals_in_doctest
 class CaseInsensitiveDict(MutableMapping):
     """A dict with case-insensitive lookup.
 
     >>> d = CaseInsensitiveDict(TesT='passed')
     >>> d
-    CaseInsensitiveDict({'TesT': 'passed'})
+    CaseInsensitiveDict({'TesT': u'passed'})
     >>> d.lower()
-    CaseInsensitiveDict({'test': 'passed'})
+    CaseInsensitiveDict({'test': u'passed'})
     >>> print(d['TesT'])
     passed
     >>> print(d['test'])
@@ -97,8 +104,8 @@ class CaseInsensitiveDict(MutableMapping):
     True
     >>> 'Test' in d
     True
-    >>> print(list(d.keys()))
-    ['Test']
+    >>> list(d.keys())
+    [u'Test']
     >>> for key in d:
     ...     print(key)
     Test
@@ -129,20 +136,20 @@ class CaseInsensitiveDict(MutableMapping):
     >>> CaseInsensitiveDict(
     ...     (key, value) for key, value in [('a', 'b')]
     ... )
-    CaseInsensitiveDict({'a': 'b'})
+    CaseInsensitiveDict({u'a': u'b'})
 
     """
 
     def __init__(self, *args, **kwargs):
         initial = dict(*args, **kwargs)
-        self._dict = dict((key.lower(), value) for key, value in iter(initial.items()))
+        self._dict = dict((key.lower(), value) for key, value in initial.items())
         self._keys = dict((key.lower(), key) for key in initial)
 
     def __len__(self):
         return len(self._dict)
 
     def __iter__(self):
-        return iter(list(self._keys.values()))
+        return iter(self._keys.values())
 
     def __setitem__(self, key, value):
         """To implement lowercase keys."""
@@ -168,11 +175,11 @@ class CaseInsensitiveDict(MutableMapping):
             type(self).__name__, repr(dct),
         )
 
-    def iteritems_lower(self):
+    def items_lower(self):
         return ((key.lower(), value) for key, value in self.items())
 
     def lower(self):
-        return type(self)(self.iteritems_lower())
+        return type(self)(self.items_lower())
 
 
 class CaseInsensitiveDefaultDict(CaseInsensitiveDict):
@@ -205,6 +212,7 @@ class CaseInsensitiveDefaultDict(CaseInsensitiveDict):
             return self.default_factory()
 
 
+@fix_unicode_literals_in_doctest
 class OrderedCaseInsensitiveDict(CaseInsensitiveDict):
     """ An (incomplete) ordered case-insensitive dict.
 
@@ -214,29 +222,29 @@ class OrderedCaseInsensitiveDict(CaseInsensitiveDict):
     ...     ('Tres', 3),
     ... ])
     >>> d
-    OrderedCaseInsensitiveDict([('Uno', 1), ('Dos', 2), ('Tres', 3)])
+    OrderedCaseInsensitiveDict([(u'Uno', 1), (u'Dos', 2), (u'Tres', 3)])
     >>> d.lower()
-    OrderedCaseInsensitiveDict([('uno', 1), ('dos', 2), ('tres', 3)])
-    >>> list(d.keys())
-    ['Uno', 'Dos', 'Tres']
-    >>> list(d.items())
-    [('Uno', 1), ('Dos', 2), ('Tres', 3)]
-    >>> list(d.values())
+    OrderedCaseInsensitiveDict([(u'uno', 1), (u'dos', 2), (u'tres', 3)])
+    >>> d.keys()
+    [u'Uno', u'Dos', u'Tres']
+    >>> d.items()
+    [(u'Uno', 1), (u'Dos', 2), (u'Tres', 3)]
+    >>> d.values()
     [1, 2, 3]
     >>> d['Cuatro'] = 4
-    >>> list(d.keys())
-    ['Uno', 'Dos', 'Tres', 'Cuatro']
-    >>> list(d.items())
-    [('Uno', 1), ('Dos', 2), ('Tres', 3), ('Cuatro', 4)]
-    >>> list(d.values())
+    >>> d.keys()
+    [u'Uno', u'Dos', u'Tres', u'Cuatro']
+    >>> d.items()
+    [(u'Uno', 1), (u'Dos', 2), (u'Tres', 3), (u'Cuatro', 4)]
+    >>> d.values()
     [1, 2, 3, 4]
     >>> list(d)
-    ['Uno', 'Dos', 'Tres', 'Cuatro']
-    >>> list(d.keys()) == list(d.keys())
+    [u'Uno', u'Dos', u'Tres', u'Cuatro']
+    >>> list(d.iterkeys()) == d.keys()
     True
-    >>> list(d.values()) == list(d.values())
+    >>> list(d.itervalues()) == d.values()
     True
-    >>> list(d.items()) == list(d.items())
+    >>> list(d.items()) == d.items()
     True
     >>> 'Uno' in d
     True
@@ -258,18 +266,18 @@ class OrderedCaseInsensitiveDict(CaseInsensitiveDict):
     4
     >>> d['UNO'] = 'one'
     >>> d['uno']
-    'one'
+    u'one'
     >>> d['Uno']
-    'one'
-    >>> list(d.keys())
-    ['Uno', 'Dos', 'Tres', 'Cuatro']
+    u'one'
+    >>> d.keys()
+    [u'Uno', u'Dos', u'Tres', u'Cuatro']
     >>> d['cuatro'] = 'four'
     >>> d['Cuatro']
-    'four'
+    u'four'
     >>> d['cuatro']
-    'four'
-    >>> list(d.keys())
-    ['Uno', 'Dos', 'Tres', 'Cuatro']
+    u'four'
+    >>> d.keys()
+    [u'Uno', u'Dos', u'Tres', u'Cuatro']
 
     """
 
@@ -322,7 +330,7 @@ class OrderedCaseInsensitiveDict(CaseInsensitiveDict):
     def values(self):
         return [self[key] for key in self.order]
 
-    def iteritems(self):
+    def items(self):
         for key in self.order:
             yield key, self[key]
 
@@ -331,10 +339,11 @@ class OrderedCaseInsensitiveDict(CaseInsensitiveDict):
 
     def __repr__(self):
         return '{0}({1})'.format(
-            type(self).__name__, repr(list(self.items()))
+            type(self).__name__, repr(self.items())
         )
 
 
+@fix_unicode_literals_in_doctest
 class CaseInsensitiveSet(MutableSet):
     """A very basic case-insensitive set.
 
@@ -344,12 +353,12 @@ class CaseInsensitiveSet(MutableSet):
     >>> 'a' in s
     False
     >>> list(CaseInsensitiveSet(['aaa', 'Aaa', 'AAA']))
-    ['aaa']
+    [u'aaa']
     >>> s = CaseInsensitiveSet(['Aaa', 'Bbb'])
     >>> s
-    CaseInsensitiveSet(['Aaa', 'Bbb'])
+    CaseInsensitiveSet([u'Aaa', u'Bbb'])
     >>> s.lower()
-    CaseInsensitiveSet(['aaa', 'bbb'])
+    CaseInsensitiveSet([u'aaa', u'bbb'])
     >>> len(s)
     2
     >>> 'aaa' in s
@@ -422,4 +431,3 @@ class CaseInsensitiveSet(MutableSet):
 
     def lower(self):
         return type(self)(self._set)
-
